# -*- coding: utf-8 -*-
#Boa:Frame:MainWindow

import os

import wx
import wx.stc
import wx.richtext

import api


# Default icon path
ICON_DIR = ['/usr/share/pixmaps', '/usr/local/share/pixmaps']
ICON_NAME = 'adaura-gui.png'

# Terminal window text colors
COLOUR_INPUT = wx.Colour(0, 90, 0)
COLOUR_DEFAULT = wx.NullColor
COLOUR_ERROR = wx.RED


def create(parent):
    return MainWindow(parent)


[wxID_MAINWINDOW, wxID_MAINWINDOWBGPANEL, wxID_MAINWINDOWBTNREFRESH, 
 wxID_MAINWINDOWBTNSEND, wxID_MAINWINDOWCOMBODEVICES, 
 wxID_MAINWINDOWLOGOBITMAP, wxID_MAINWINDOWLOGOPANEL, 
 wxID_MAINWINDOWTEXTINPUT, wxID_MAINWINDOWTEXTOUTPUT, 
] = [wx.NewId() for _init_ctrls in range(9)]


class MainWindow(wx.Frame):
    def _init_ctrls(self, prnt):
        # generated method, don't edit
        wx.Frame.__init__(self, id=wxID_MAINWINDOW, name=u'MainWindow',
              parent=prnt, pos=wx.Point(418, 331), size=wx.Size(640, 480),
              style=wx.DEFAULT_FRAME_STYLE, title=u'Serial Port')
        self.SetClientSize(wx.Size(640, 480))
        self.SetMaxSize(wx.Size(-1, -1))
        self.Center(wx.BOTH)
        self.Bind(wx.EVT_CLOSE, self.OnMainWindowClose)

        self.bgPanel = wx.Panel(id=wxID_MAINWINDOWBGPANEL, name='panel1',
              parent=self, pos=wx.Point(0, 0), size=wx.Size(640, 480),
              style=wx.TAB_TRAVERSAL)

        self.logoPanel = wx.Panel(id=wxID_MAINWINDOWLOGOPANEL,
              name=u'logoPanel', parent=self.bgPanel, pos=wx.Point(0, 0),
              size=wx.Size(640, 68), style=wx.TAB_TRAVERSAL)
        self.logoPanel.SetBackgroundColour(wx.Colour(0, 0, 0))

        self.logoBitmap = wx.StaticBitmap(bitmap=wx.NullBitmap,
              id=wxID_MAINWINDOWLOGOBITMAP, name=u'logoBitmap',
              parent=self.logoPanel, pos=wx.Point(195, 0), size=wx.Size(250,
              68), style=0)
        self.logoBitmap.Center(wx.HORIZONTAL)

        self.textOutput = wx.TextCtrl(id=wxID_MAINWINDOWTEXTOUTPUT,
              name=u'textOutput', parent=self.bgPanel, pos=wx.Point(8, 72),
              size=wx.Size(624, 328), style=wx.TE_MULTILINE|wx.TE_RICH, value=u'')
        self.textOutput.SetEditable(False)
        self.textOutput.SetToolTipString(u'Output window')

        self.comboDevices = wx.ComboBox(choices=[],
              id=wxID_MAINWINDOWCOMBODEVICES, name=u'comboDevices',
              parent=self.bgPanel, pos=wx.Point(8, 411), size=wx.Size(496, 27),
              style=0, value=u'')
        self.comboDevices.SetLabel(u'')
        self.comboDevices.Bind(wx.EVT_COMBOBOX, self.OnComboDevicesSelect,
              id=wxID_MAINWINDOWCOMBODEVICES)

        self.btnRefresh = wx.Button(id=wxID_MAINWINDOWBTNREFRESH,
              label=u'Refresh list', name=u'btnRefresh', parent=self.bgPanel,
              pos=wx.Point(514, 411), size=wx.Size(118, 27), style=0)
        self.btnRefresh.Bind(wx.EVT_BUTTON, self.OnBtnRefreshClick,
              id=wxID_MAINWINDOWBTNREFRESH)

        self.textInput = wx.TextCtrl(id=wxID_MAINWINDOWTEXTINPUT,
              name=u'textInput', parent=self.bgPanel, pos=wx.Point(8, 443),
              size=wx.Size(496, 27), style=wx.TE_PROCESS_ENTER, value=u'')
        self.textInput.Bind(wx.EVT_TEXT_ENTER, self.OnTextInputEnter,
              id=wxID_MAINWINDOWTEXTINPUT)

        self.btnSend = wx.Button(id=wxID_MAINWINDOWBTNSEND, label=u'Send',
              name=u'btnSend', parent=self.bgPanel, pos=wx.Point(514, 443),
              size=wx.Size(118, 27), style=0)
        self.btnSend.Bind(wx.EVT_BUTTON, self.OnBtnSendClick,
              id=wxID_MAINWINDOWBTNSEND)

    def __init__(self, parent):
        self._init_ctrls(parent)
        self._init_events()

        # Load window icon
        for path in ICON_DIR:
            path = path + '/' + ICON_NAME

            if os.path.exists(path):
                icon = wx.EmptyIcon()
                icon.CopyFromBitmap(wx.Bitmap(path, wx.BITMAP_TYPE_ANY))
                self.SetIcon(icon)
                break

        # Load logo bitmap
        path = os.path.join(
            os.path.dirname(os.path.realpath(__file__)),
            'extras',
            'logo.jpg')

        logo = wx.Image(path, wx.BITMAP_TYPE_JPEG)
        self.logoBitmap.SetBitmap(wx.BitmapFromImage(logo))

        self.FillComboDevices()

        self.isConnected = False
        self.newline = api.NEWLINE_CRLF

        self.thread = None
        self.serial = api.CreateSerial()

    def _init_events(self):
        self.Bind(api.EVT_RECEIVED, self.OnBytesReceived)
        self.Bind(api.EVT_READERROR, self.OnReadError)

    def OnBytesReceived(self, event):
        self.Write(event.data, False)

    def OnReadError(self, event):
        self.WriteError(event.data)
        self.CloseConnection()

    def OnBtnSendClick(self, event):
        if self.isConnected:
            text = self.textInput.GetValue()

            self.WriteUser(text)

            try:
                api.WriteLine(self.serial, text)
            except Exception as e:
                self.WriteError(str(e))
                self.CloseConnection()
        else:
            self.Write('No connection to port')

        self.textInput.Clear()

    def OnTextInputEnter(self, event):
        self.OnBtnSendClick(event)

    def OnBtnRefreshClick(self, event):
        self.comboDevices.Clear()
        self.FillComboDevices()

    def OnComboDevicesSelect(self, event):
        """
        If a user has selected some element of the combobox,
        we need to automatically connect to the selected port
        """
        if self.isConnected:
            self.CloseConnection()

        port = self.comboDevices.GetValue()
        self.serial.port = port

        try:
            self.serial.open()
        except Exception as e:
            self.WriteError(str(e))
        else:
            try:
                # Start background reader thread (to prevent GUI blocking)
                self.thread = api.InputThread(
                    parent=self,
                    serial=self.serial,
                    newline=self.newline)

                self.thread.start()
            except Exception as e:
                self.serial.close()
                self.WriteError(str(e))
            else:
                self.WriteUser('Connected to {}'.format(port))
                self.isConnected = True

    def OnMainWindowClose(self, event):
        if self.isConnected:
            self.CloseConnection()

        self.Destroy()

    def FillComboDevices(self):
        """
        Shortcut function
        """
        self.comboDevices.AppendItems(api.GetPortsList())

        # Uncomment when debugging with Socat:
        # self.comboDevices.AppendItems(["/dev/pts/9"])

    def CloseConnection(self):
        """
        Shortcut function
        """
        self.thread.abort()
        self.thread.join()
        self.thread = None

        self.serial.close()  
        self.isConnected = False

    def Write(self, text, line=True):
        """
        Shortcut function
        """
        if line:
            text += os.linesep

        self.textOutput.AppendText(text)

    def WriteUser(self, text):
        """
        Shortcut function
        """
        self.textOutput.SetForegroundColour(COLOUR_INPUT)
        self.Write("# " + text)
        self.textOutput.SetForegroundColour(COLOUR_DEFAULT)

    def WriteError(self, text):
        """
        Shortcut function
        """
        self.textOutput.SetForegroundColour(COLOUR_ERROR)
        self.Write(text)
        self.textOutput.SetForegroundColour(COLOUR_DEFAULT)
